/* main.vala
 *
 * Copyright (C) 2008-2009 Nicolas Joseph
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Nicolas Joseph <nicolas.joseph@valaide.org>
 */

/*string encoding_charset;*/
/*bool new_window_option;*/
bool new_document_option;
bool show_version;
string[] remaining_args;

const OptionEntry[] options = {
{ "version", 'V', 0, OptionArg.NONE, ref show_version, N_("Show the application's version"), null },
/* { "encoding", '\0', 0, OptionArg.STRING, ref encoding_charset, N_("Set the character encoding to be used to open the files listed on the command line"), N_("ENCODING")}, */
/*{ "new-window", '\0', 0, OptionArg.NONE, ref new_window_option, N_("Create a new toplevel window in an existing instance of valide"), null },*/
{ "new-document", '\0', 0, OptionArg.NONE, ref new_document_option, N_("Create a new document in an existing instance of valide"), null },
/* "" = G_OPTION_REMAINING */
{ "", '\0', 0, OptionArg.FILENAME_ARRAY, ref remaining_args, null, N_("[FILE...]") }, /* collects file arguments */
{null}
};

public class Valide.Application : Object
{
  private List<string> _file_list;
  private HashTable<string, Window> windows;
 
  public List<string> file_list
  {
    get
    {
      return this._file_list.first ();
    }
    private set
    {
      this._file_list = value.copy ();
    }
  }

  private void message (Unique.App sender, Unique.Command command,
                        string data, string startup_id, uint workspace)
  {
    string ws;
    Window window;
    bool opened = false;

    ws = workspace.to_string ();
    window = this.windows.lookup (ws);
    if (window == null)
    {
      window = this.create_new_window (workspace);
      this.windows.insert (ws, window);
    }
    switch (command)
    {
      case Unique.Command.NEW:
        window.documents.create ();
        opened = true;
      break;
      case Unique.Command.OPEN:
        if (data != "")
        {

          try
          {
            if (window.projects.accept_file (data))
            {
              window.projects.open (data);
            }
            else
            {
              window.documents.create (data);
            }
            opened = true;
          }
          catch (Error e)
          {
            debug (e.message);
          }
        }
      break;
    }
    if (opened)
    {
      window.hide_start_page ();
    }
  }

  private Window create_new_window (uint workspace)
  {
    Window window;

    window = new Window ();
    window.workspace = workspace;
    window.destroy.connect ((s) => {
      string ws;

      ws = (s as Window).workspace.to_string ();
      this.windows.remove (ws);
      if (this.windows.size () == 0)
      {
        Gtk.main_quit ();
        Xml.Parser.cleanup ();
      }
    });

    window.run ();
    return window;
  }

  public Application (string[] args)
  {
    OptionContext context;

    Xml.Parser.init ();
    Gtk.init (ref args);
    Intl.bindtextdomain (Config.GETTEXT_PACKAGE, Config.LOCALE_DIR);
    Intl.bind_textdomain_codeset (Config.GETTEXT_PACKAGE, "UTF-8");

    this.windows = new HashTable<string, Window> (str_hash, str_equal);

    /* Setup command line options */
    context = new OptionContext (_("- Edit text files"));
    context.add_main_entries (options, Config.GETTEXT_PACKAGE);
    context.add_group (Gtk.get_option_group (false));

    try
    {
      context.parse (ref args);
      if (remaining_args != null)
      {
        this.file_list = new List<string> ();
        for (int i = 0; remaining_args[i] != null; i++)
        {
          this._file_list.append (remaining_args[i]);
        }
      }
    }
    catch (OptionError e)
    {
      print (_("%s\nRun '%s --help' to see a full list of available command line options.\n"),
             e.message, args[0]);
    }
  }

#if (!DBUS)
  public int run ()
  {
    if (show_version)
    {
      print ("%s - Version %s\n", Environment.get_application_name (),
                                  Config.VERSION);
    }
    else
    {
      Window window;

      window = this.create_new_window (0);
      Gtk.main ();
    }
    return 0;
  }
#else
  public int run ()
  {
    Unique.App app;

    if (show_version)
    {
      print ("%s - Version %s\n", Environment.get_application_name (),
                                  Config.VERSION);
    }
    else
    {
      app = new Unique.App ("valaide");
      if (app.is_running ())
      {
        if (new_document_option)
        {
          app.new_document ();
        }
        else if (this.file_list != null)
        {
          foreach (string uri in this.file_list)
          {
            app.open_uri (uri);
          }
        }
        else
        {
          app.activate ();
        }
      }
      else
      {
        if (new_document_option)
        {
          this.message (app, Unique.Command.NEW, "", app.startup_id,
                        app.workspace);
        }
        else if (this.file_list != null)
        {
          foreach (string uri in this.file_list)
          {
            this.message (app, Unique.Command.OPEN, uri, app.startup_id,
                          app.workspace);
          }
        }
        else
        {
          this.message (app, Unique.Command.ACTIVATE, "", app.startup_id,
                        app.workspace);
        }
        app.message.connect (message);
        Gtk.main ();
      }
    }
    return 0;
  }
#endif
}

int main (string[] args)
{
  return new Valide.Application (args).run ();
}

